/**********************************************************************************
!C

!File: bin2dec.c

!Description:
  convert a binary number to decimal 

!Input Parameters: (none)

!Input/Output Parameters: (none)

!Output Parameters: (none)

!Revision History:
  Original July 1999. QA Tool Version 1.2
  Original Sept 2001. QA Tool Version 2.0 
		      (Fixed bug, and tested for Linux/IRIX)

!Team-unique Header:
  This software was developed by:
    Land Data Operational Product Evaluation (LDOPE) Team for the
    National Aeronautics and Space Administration, Goddard Space Flight
    Center

!References and Credits:
    Sadashiva Devadiga
    LDOPE                             Science Systems and Applications Inc.
    devadiga@ltpmail.gsfc.nasa.gov    NASA/GSFC Code 922
    phone: 301-614-5449               Greenbelt, MD 20771

!Design Notes: (none)

!END
*********************************************************************************/

#include <stdio.h>
#include <string.h>

#define HELP \
"NAME \n" \
"       bin2dec - convert a signed binary number to decimal \n" \
"\n" \
"SYNOPSIS \n" \
"       bin2dec [-help] \n" \
"       bin2dec <number> \n" \
" \n" \
"       echo <number> | bin2dec \n" \
" \n" \
"DESCRIPTION \n" \
"       This routine converts a signed binary number to equivalent decimal. \n" \
"The binary number is considered to be -ve if the most significant bit is a 1\n" \
" \n" \
"OPTIONS \n" \
"       -help                       Display this help message \n" \
"       number                      Number in binary \n" \
" \n" \
"Examples: \n" \
"	bin2dec 10110 \n" \
" \n" \
"Last update: Version 3.6, 02/12/2020 \n" \
"Please report problems to Sadashiva Devadiga (sadashiva.devadiga-1@nasa.gov) \n"

#define USAGE \
"usage:		bin2dec <number> \n" \
" \n" \
"OPTIONS \n" \
"       -help                       Display this help message \n" \
"       number                      Number in binary \n" \
" \n"

static unsigned int BIT[] = {
  0x1,        0x2,        0x4,        0x8,
  0x10,       0x20,       0x40,       0x80,
  0x100,      0x200,      0x400,      0x800,
  0x1000,     0x2000,     0x4000,     0x8000,
  0x10000,    0x20000,    0x40000,    0x80000,
  0x100000,   0x200000,   0x400000,   0x800000,
  0x1000000,  0x2000000,  0x4000000,  0x8000000,
  0x10000000, 0x20000000, 0x40000000, 0x80000000
};

int main(int argc, char **argv)
{
  char c;
  int st = 1;
  int i, len;
  int sum, num_sgn;
  char num_str[35];

  if ((argc == 2) && ((strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "-help") == 0)))
    fprintf(stderr, "%s\n", HELP);
  else if (argc != 2)
  {
    fprintf(stderr, "Usage error . . . .  \n\n");
    fprintf(stderr, "%s\n", USAGE);
  }
  else
  {
    len = (int)strlen(argv[1]) - 1;
    if (argv[1][0] == '1')
    {
      num_sgn = 1;
      for (i=0; i<=len; i++)
      {
        if (argv[1][i] == '0') argv[1][i] = '1';
	else if (argv[1][i] == '1') argv[1][i] = '0';
	else { st = -1; break; }
      }
    }
    else if (argv[1][0] == '0') num_sgn = 0;
    else st = -1;
    if (st == 1)
    {
      for (i=len, sum=0; i>=0; i--)
      {
        c = argv[1][i];
        if (c == '0') continue;
        else if (c == '1') sum += BIT[len-i] ;
	else { st = -1; break; }
      }
      if (st != -1)
      {
        if (num_sgn == 1) printf("-%d\n", sum+1);
        else printf("%d\n", sum);
      }
    }
    if (st == -1) fprintf(stderr, "Not a valid binary number: %s \n", argv[1]);
  }
  return 0;
}


