/*
!C-INC************************************************************************

File: sds_rw.h

!Description:
  SDS read write libray headers - contains structures, types and function
declarations

!Input Parameters: (none)

!Input/output Parameters: (none)

!Output Parameters: (none)

!Revision History:
  Original April 2002. Version 2.0.0

!Team-unique Header:
  This software was developed by:
    Modis Science Team and Land Data Operational Product Evaluation (LDOPE)
    Team for the National Aeronautics and Space Administration, Goddard Space
    Flight Center

!References and Credits:
    MODIS Science Team Members:
      Chris Justice
      MODIS Land Science Team           University of Maryland
      justice@kratmos.gsfc.nasa.gov     NASA/GSFC Code 923
      phone: 301-286-7372               Greenbelt, MD 20771

      David Roy
      LDOPE                             University of Maryland
      droy@kratmos.gsfc.nasa.gov        NASA/GSFC Code 922
      phone: 301 614 5571               Greenbelt, MD 20771

    Developers:
      Sadashiva Devadiga
      LDOPE                             Science Systems and Applications Inc.
      devadiga@ltpmail.gsfc.nasa.gov    NASA/GSFC Code 922
      phone: 301-614-5449               Greenbelt, MD 20771

!Design Notes: (none)

!END
**************************************************************************/
#include "sds_types.h"
#include "hdf.h"

#define MAX_ATTR_LENGTH 80 
#define QA_SDS_FILL 255
#define QA_SDS_DATA_TYPE DFNT_UINT8
#define QA_SDS_SHORT_NAME "surf_refl_state_qa"
#define QA_SDS_LONG_NAME "Surface Reflectance State QA"
#define NBUF 16 
#define HDF_ERROR -1
#define HDFEOSgridname_hkm_terra "MOD_GRID_HDFSR"
#define HDFEOSgridname_qkm_terra "MOD_GRID_QDFSR"
#define HDFEOSgridname_hkm_aqua "MYD_GRID_HDFSR"
#define HDFEOSgridname_qkm_aqua "MYD_GRID_QDFSR"

/* Sudipta-09/16: HDF5 addition */
typedef struct {
        char name[255];    /* MSZ changed from 100 to 255 in case a very long name for HDF5 absolute sds name */
        char abs_gnames[255]; /* To keep track of the absolute group name for H5, SS-11/15  It is safer not to use **abs_gnames because overlapped calloc */
        char **h5sdsnames; /* MSZ, currently don't use it but just one sds name have one group name, abs_gnames */
        hid_t h5space_id;
        hid_t rank;
        hsize_t *h5_dims;
        int sds_found;
        int srch_type; /* 0=name check, 1=sds count, 2=non1d sds count, 3=sds names */
        int nsds;
}H5SDS;

typedef struct {
    hid_t loc_id;
    void **rdata;
    char name[200];
} H5_ATTR;

/* struct definition, MSZ 8/15/17 */
typedef struct
{
    uint8 endian;           /* 1 is big endian and 0 little endian */
    hid_t num_type;         /* for numerical type of H5T_INTEGER and H5T_FLOAT, e.g. H5T_STD_I16BE and H5T_IEEE_F32BE, for non-numerical, it is -1 */
    int HDF5_type;          /* predefined number for data type, e.g. DFNT_INT32 */
    H5T_class_t type_class; /* original H5T_class_t */
    size_t size;            /* number of bytes for each datum. Note that for H5T_C_S1, size is same as the length of the string. */
    H5T_sign_t sign;        /* sign or unsigned for integer data types */
}HDF5_DATA_TYPE_INFO;

/* Error messages */
typedef struct {
  int num;              /* error number */
  char *message;        /* error message */
  char *moreinfo;       /* more information about the error */
} error_t;

char *get_attr_metadata(char *in_fname, char *meta_str);
int get_sds_info(const char *hdf_fname, sds_t *sds_info);
void get_sds_dim_name(sds_t *sds_info, char **dim_names, char **short_dim_names);
int get_sds_data(sds_t *sds, void *data);
void *get_sel_sds_data(sds_t *sds, int32 *start, int32 *edge);
int read_sel_sds_data(sds_t *sds_info, int32 *start, int32 *edge, void *data);
int set_sel_sds_data(sds_t *sds_info, int32 *start, int32 *edge, void *data);

/* MSZ 1/22/2020 */
int set_sel_sds_data_compression(sds_t *sds_info, int32 *start, int32 *edge, int comp_chunk_rows, void *data);

int set_sel_l2g_sds_2D_data_chunked_gzip(sds_t *sds_info, int32 *start, int32 *edge, void *data);
int open_sd(char *fname, sds_t *sds_info, char open_t);
int open_sds(char *fname, sds_t *sds_info, char open_t);

/* MSZ 1/22/2020 */
int open_sds_compression(char *fname, sds_t *sds_info, int comp_chunk_rows, char open_t);

void close_hdf(sds_t *sds_info, int H5IO);
void close_sds(sds_t *sds_info, int H5IO);
void close_sd(sds_t *sds_info, int H5IO);
int get_l2g_sds_names(char *fname, char **sds_names);
int get_H5_full_sds_names(char *fname, char **sds_names);
int get_sds_names(char *fname, char **sds_names);
int get_sds_names_exclusions(char *fname, char **sds_names, char **sds_names_exclusion, int num_names_exclusion);
int get_nsds(char *fname);
int sds_name_existence(char *fname, char *sds_name);
int sds_name_existence_soft(char *fname, char *sds_name);
int get_sd_attr_names(const char *HDF_fname, char **attr_names);
int get_sd_attr_num(const char *HDF_fname);
int file_attr_name_existence(char *fname, char *attr_name);
void *get_sd_attr(sds_t *sds_info, char *attr_name, int32 *attr_type, int32 *attr_cnt);
void *find_get_sd_attr(sds_t *sds_info, char *attr_name, int32 *attr_type, int32 *attr_cnt);
void find_get_sd_attr_to_str(char *fname, char *attr_name, char *meta_val);
void *get_sds_attr(sds_t *sds_info, char *attr_name, int32 *attr_type, int32 *attr_cnt);
void write_attr_fval(int H5OUT, int32 sds_id, int32 fval_type, int c, int attr_val);
int set_sd_attr(int H5OUT, int out_sd_id, char *attr_name, int attr_data_type, int n_values, void *values);
int set_sds_attr(int H5OUT, int out_sds_id, char *attr_name, int attr_data_type, int n_values, void *values);
void write_all_sds_attr(int H5IN, int32 in_sds_id, int H5OUT, int32 out_sds_id, int32 bn);
void write_metadata(int H5IN, int32 in_sd_id, int H5OUT, int32 out_sd_id);
void compute_sds_start_offset(sds_t *sds_info, int n, int m, int *st_c, int *offset);
void update_nd_sdsnames(char **sds_names, int *sds_cnt, char *fname);
void update_l2g_sdsnames(char **sds_names, int *nsds, char *fname, int nobs);
void create_names(char *fname, char *sds_name, char *nd_ext, char *md_ext, int *nsds, char **sds_names);
void get_dim_num(char *str_arr, int *num_arr, int dim_size, int *cnt);
void copy_sds(sds_t *sds_info, int32 out_sd_id, int compress);
void check_and_fix_sdsname(char *hdf_fname, sds_t *sds_info);
int compute_sds_ndata(sds_t *sds_info);
void get_sds_edge(sds_t *sds_info, int32 *edge);
void compute_sds_nrows_ncols(sds_t *sds_info, int *nrows, int *ncols);
void get_sds_param(sds_t *sds_info, int *n, int *m, int *rank, int *dim_size);
void display_sds_info_of_file(char* filename);
void writeMetadata(int H5IN, int32 in_sd_id, int H5OUT, int32 out_sd_id);
void write_attr_fvalf(int H5OUT, int32 sds_id, int32 fval_type, int c, double attr_val);
void write_attr_fsoval(char *fname, int32 sds_id, int32 so_type, float64 attr_sval, float64 attr_oval);
int HDF_check_L2G(const char*, int*);
int HDF_version_check(const char *HDF_name);
int L2G_format_check(char *fname);

/* MSZ 1/16/2020, for speed-up reduce_an_sds_by_sub() */
int get_lines_from_chunk(void *data_chunk, int data_rank, int chunk_shape[], int bandinterleave, int element_size, int lines_start, int n_lines, void *data_lines);

/* HDF5 functions */
herr_t H5Fget_objs_links(hid_t fid, H5_OBJS_TYPES *h5_obj_types);
herr_t H5lnk_handler (hid_t loc_id, const char *name, const H5L_info_t *info,void *sds);
herr_t H5_attr_write_hdf4(hid_t obj_id, const char *attr_name, const H5A_info_t *ainfo,
    void *op_data);
herr_t H5_attr_write_hdf5(hid_t obj_id, const char *attr_name, const H5A_info_t *ainfo,
    void *op_data);
herr_t H5_attr_read(hid_t obj_id, const char *attr_name, const H5A_info_t *ainfo,
    void *op_data);
herr_t H5dataset_search (hid_t loc_id, const char *name, const H5O_info_t *info,
            void *sds);
int32 H5toH4type(hid_t h5type);
int convH4toH5 (const int32 h4type, hid_t* h5memtype, size_t* h4memsize,
				size_t* h4size, hid_t *h5type);
int setH5Attr(int32 att_type, char *att_name, int nval,void *mbuf, hid_t id);
hid_t mkstr( size_t size, H5T_str_t pad);
H5SDS *h5_dsetget(sds_t * sds_info, char *sdsname);
int h5_dsetinfo(H5SDS *h5_sds, sds_t *sds_info);
int HDF5_split_abs_name_into_path_name(const char *obj_abs_name, char **abs_path, char **name);
HDF5_DATA_TYPE_INFO HDF5_get_data_type_info(hid_t tid);
