#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "qa_tool.h"
#include "tile_bound_info.h"

int read_bound_table(int hn, int vn, char* proj_name, double *lon_min, double *lon_max,
                     double *lat_min, double *lat_max)
/****************************************************************************
!C

!File: tile_bound_info.c

!Description:
  Routine to find tile bondary information. 

!Input Parameters: (none)

!Input/Output Parameters: (none)

!Output Parameters: (none)

!Revision History:
  Version 1.0    December 2003

!Team-unique Header:
  This software was developed by:
    Land Data Operational Product Evaluation (LDOPE) Team for the
    National Aeronautics and Space Administration, Goddard Space Flight
    Center

!References and Credits:

    Sadashiva Devadiga
    LDOPE                             Science Systems and Applications Inc.
    devadiga@ltpmail.gsfc.nasa.gov    NASA/GSFC Code 922
    phone: 301-614-5449               Greenbelt, MD 20771

    Yi (Sophyi) Zhang
    LDOPE                             Science Systems and Applications Inc.
    sophyi@ltpmail.gsfc.nasa.gov      NASA/GSFC Code 922
    phone: 301-614-5497               Greenbelt, MD 20771

    David Roy
    LDOPE                             University of Maryland, Department of Geography
    droy@kratmos.gsfc.nasa.gov        NASA/GSFC Code 922 (B32)
    phone: 301-614-5571               Greenbelt, MD 20771

!Design Notes: (none)

!END
*****************************************************************************/
{
  FILE *fp;
  int in_tile_hn, in_tile_vn;
  int tile_find, c;
  double c1, c2, c3, c4;
  char *anc_path;
  char table_fname[MAX_PATH_LENGTH];
  
  anc_path = getenv("ANCPATH");
  if (anc_path == NULL)
    {
      fprintf(stderr, "Please set up your environment variable for ANCPATH first.\n");
      exit(EXIT_FAILURE);
    }

  if (strcmp(proj_name, "ISIN") == 0)
    {
      sprintf(table_fname, "%s/%s", anc_path, ISIN_TABLE_FNAME);
    }
  else if (strcmp(proj_name, "SIN") == 0)
    {
      sprintf(table_fname, "%s/%s", anc_path, SIN_TABLE_FNAME);
    }
  else
    {
      fprintf(stderr, "No valid projection parameter entered. \n");
      exit(EXIT_FAILURE);
    }
  if ((fp = fopen(table_fname, "r")) == NULL)
    fprintf(stderr, "Cannot open projection boundary file %s\n", table_fname);
  else
  {
    tile_find = 0;
    c = fgetc(fp);
    while (c != EOF)
    {
      ungetc(c, fp);
      fscanf(fp, "%d %d", &in_tile_vn, &in_tile_hn);
      fscanf(fp, "%lf %lf %lf %lf", &c1, &c2, &c3, &c4);
      if ((in_tile_vn == vn) && (in_tile_hn == hn))
        {
          *lon_min = c1;
          *lon_max = c2;
          *lat_min = c3;
          *lat_max = c4;
          tile_find = 1;
          break;
        }
      c = fgetc(fp);
    }
    fclose(fp);
  }
  return tile_find;
}


